@@ -67,6 +67,7 @@ struct mkfs_allocation {
 static int create_metadata_block_groups(struct btrfs_root *root, int mixed,
 				struct mkfs_allocation *allocation)
 {
+	struct btrfs_fs_info *fs_info = root->fs_info;
 	struct btrfs_trans_handle *trans;
 	u64 bytes_used;
 	u64 chunk_start = 0;
@@ -74,10 +75,10 @@ static int create_metadata_block_groups(struct btrfs_root *root, int mixed,
 	int ret;
 
 	trans = btrfs_start_transaction(root, 1);
-	bytes_used = btrfs_super_bytes_used(root->fs_info->super_copy);
+	bytes_used = btrfs_super_bytes_used(fs_info->super_copy);
 
 	root->fs_info->system_allocs = 1;
-	ret = btrfs_make_block_group(trans, root, bytes_used,
+	ret = btrfs_make_block_group(trans, fs_info, bytes_used,
 				     BTRFS_BLOCK_GROUP_SYSTEM,
 				     BTRFS_FIRST_CHUNK_TREE_OBJECTID,
 				     0, BTRFS_MKFS_SYSTEM_GROUP_SIZE);
@@ -86,7 +87,7 @@ static int create_metadata_block_groups(struct btrfs_root *root, int mixed,
 		return ret;
 
 	if (mixed) {
-		ret = btrfs_alloc_chunk(trans, root->fs_info->extent_root,
+		ret = btrfs_alloc_chunk(trans, fs_info,
 					&chunk_start, &chunk_size,
 					BTRFS_BLOCK_GROUP_METADATA |
 					BTRFS_BLOCK_GROUP_DATA);
@@ -96,7 +97,7 @@ static int create_metadata_block_groups(struct btrfs_root *root, int mixed,
 		}
 		if (ret)
 			return ret;
-		ret = btrfs_make_block_group(trans, root, 0,
+		ret = btrfs_make_block_group(trans, fs_info, 0,
 					     BTRFS_BLOCK_GROUP_METADATA |
 					     BTRFS_BLOCK_GROUP_DATA,
 					     BTRFS_FIRST_CHUNK_TREE_OBJECTID,
@@ -105,7 +106,7 @@ static int create_metadata_block_groups(struct btrfs_root *root, int mixed,
 			return ret;
 		allocation->mixed += chunk_size;
 	} else {
-		ret = btrfs_alloc_chunk(trans, root->fs_info->extent_root,
+		ret = btrfs_alloc_chunk(trans, fs_info,
 					&chunk_start, &chunk_size,
 					BTRFS_BLOCK_GROUP_METADATA);
 		if (ret == -ENOSPC) {
@@ -114,7 +115,7 @@ static int create_metadata_block_groups(struct btrfs_root *root, int mixed,
 		}
 		if (ret)
 			return ret;
-		ret = btrfs_make_block_group(trans, root, 0,
+		ret = btrfs_make_block_group(trans, fs_info, 0,
 					     BTRFS_BLOCK_GROUP_METADATA,
 					     BTRFS_FIRST_CHUNK_TREE_OBJECTID,
 					     chunk_start, chunk_size);
@@ -134,12 +135,13 @@ static int create_data_block_groups(struct btrfs_trans_handle *trans,
 		struct btrfs_root *root, int mixed,
 		struct mkfs_allocation *allocation)
 {
+	struct btrfs_fs_info *fs_info = root->fs_info;
 	u64 chunk_start = 0;
 	u64 chunk_size = 0;
 	int ret = 0;
 
 	if (!mixed) {
-		ret = btrfs_alloc_chunk(trans, root->fs_info->extent_root,
+		ret = btrfs_alloc_chunk(trans, fs_info,
 					&chunk_start, &chunk_size,
 					BTRFS_BLOCK_GROUP_DATA);
 		if (ret == -ENOSPC) {
@@ -148,7 +150,7 @@ static int create_data_block_groups(struct btrfs_trans_handle *trans,
 		}
 		if (ret)
 			return ret;
-		ret = btrfs_make_block_group(trans, root, 0,
+		ret = btrfs_make_block_group(trans, fs_info, 0,
 					     BTRFS_BLOCK_GROUP_DATA,
 					     BTRFS_FIRST_CHUNK_TREE_OBJECTID,
 					     chunk_start, chunk_size);
@@ -244,11 +246,12 @@ static int create_one_raid_group(struct btrfs_trans_handle *trans,
 			      struct mkfs_allocation *allocation)
 
 {
+	struct btrfs_fs_info *fs_info = root->fs_info;
 	u64 chunk_start;
 	u64 chunk_size;
 	int ret;
 
-	ret = btrfs_alloc_chunk(trans, root->fs_info->extent_root,
+	ret = btrfs_alloc_chunk(trans, fs_info,
 				&chunk_start, &chunk_size, type);
 	if (ret == -ENOSPC) {
 		error("not enough free space to allocate chunk");
@@ -257,7 +260,7 @@ static int create_one_raid_group(struct btrfs_trans_handle *trans,
 	if (ret)
 		return ret;
 
-	ret = btrfs_make_block_group(trans, root->fs_info->extent_root, 0,
+	ret = btrfs_make_block_group(trans, fs_info, 0,
 				     type, BTRFS_FIRST_CHUNK_TREE_OBJECTID,
 				     chunk_start, chunk_size);
 
@@ -451,7 +454,7 @@ static int fill_inode_item(struct btrfs_trans_handle *trans,
 			   struct btrfs_inode_item *dst, struct stat *src)
 {
 	u64 blocks = 0;
-	u64 sectorsize = root->sectorsize;
+	u64 sectorsize = root->fs_info->sectorsize;
 
 	/*
 	 * btrfs_inode_item has some reserved fields
@@ -652,7 +655,7 @@ static int add_file_items(struct btrfs_trans_handle *trans,
 	u64 bytes_read = 0;
 	struct btrfs_key key;
 	int blocks;
-	u32 sectorsize = root->sectorsize;
+	u32 sectorsize = root->fs_info->sectorsize;
 	u64 first_block = 0;
 	u64 file_pos = 0;
 	u64 cur_bytes;
@@ -753,7 +756,7 @@ again:
 		if (ret)
 			goto end;
 
-		ret = write_and_map_eb(root, eb);
+		ret = write_and_map_eb(root->fs_info, eb);
 		if (ret) {
 			error("failed to write %s", path_name);
 			goto end;
@@ -798,7 +801,7 @@ static char *make_path(const char *dir, const char *name)
 
 static int traverse_directory(struct btrfs_trans_handle *trans,
 			      struct btrfs_root *root, const char *dir_name,
-			      struct directory_name_entry *dir_head, int out_fd)
+			      struct directory_name_entry *dir_head)
 {
 	int ret = 0;
 
@@ -984,6 +987,7 @@ static int create_chunks(struct btrfs_trans_handle *trans,
 			 u64 size_of_data,
 			 struct mkfs_allocation *allocation)
 {
+	struct btrfs_fs_info *fs_info = root->fs_info;
 	u64 chunk_start;
 	u64 chunk_size;
 	u64 meta_type = BTRFS_BLOCK_GROUP_METADATA;
@@ -993,11 +997,11 @@ static int create_chunks(struct btrfs_trans_handle *trans,
 	int ret;
 
 	for (i = 0; i < num_of_meta_chunks; i++) {
-		ret = btrfs_alloc_chunk(trans, root->fs_info->extent_root,
+		ret = btrfs_alloc_chunk(trans, fs_info,
 					&chunk_start, &chunk_size, meta_type);
 		if (ret)
 			return ret;
-		ret = btrfs_make_block_group(trans, root->fs_info->extent_root, 0,
+		ret = btrfs_make_block_group(trans, fs_info, 0,
 					     meta_type, BTRFS_FIRST_CHUNK_TREE_OBJECTID,
 					     chunk_start, chunk_size);
 		allocation->metadata += chunk_size;
@@ -1010,11 +1014,11 @@ static int create_chunks(struct btrfs_trans_handle *trans,
 	if (size_of_data < minimum_data_chunk_size)
 		size_of_data = minimum_data_chunk_size;
 
-	ret = btrfs_alloc_data_chunk(trans, root->fs_info->extent_root,
+	ret = btrfs_alloc_data_chunk(trans, fs_info,
 				     &chunk_start, size_of_data, data_type, 0);
 	if (ret)
 		return ret;
-	ret = btrfs_make_block_group(trans, root->fs_info->extent_root, 0,
+	ret = btrfs_make_block_group(trans, fs_info, 0,
 				     data_type, BTRFS_FIRST_CHUNK_TREE_OBJECTID,
 				     chunk_start, size_of_data);
 	allocation->data += size_of_data;
@@ -1025,8 +1029,7 @@ static int create_chunks(struct btrfs_trans_handle *trans,
 	return ret;
 }
 
-static int make_image(const char *source_dir, struct btrfs_root *root,
-		int out_fd)
+static int make_image(const char *source_dir, struct btrfs_root *root)
 {
 	int ret;
 	struct btrfs_trans_handle *trans;
@@ -1044,7 +1047,7 @@ static int make_image(const char *source_dir, struct btrfs_root *root,
 	INIT_LIST_HEAD(&dir_head.list);
 
 	trans = btrfs_start_transaction(root, 1);
-	ret = traverse_directory(trans, root, source_dir, &dir_head, out_fd);
+	ret = traverse_directory(trans, root, source_dir, &dir_head);
 	if (ret) {
 		error("unable to traverse directory %s: %d", source_dir, ret);
 		goto fail;
@@ -1406,7 +1409,7 @@ int main(int argc, char **argv)
 	u32 sectorsize = 4096;
 	u32 stripesize = 4096;
 	int zero_end = 1;
-	int fd;
+	int fd = -1;
 	int ret;
 	int i;
 	int mixed = 0;
@@ -1492,12 +1495,12 @@ int main(int argc, char **argv)
 					error("unrecognized filesystem feature '%s'",
 							tmp);
 					free(orig);
-					exit(1);
+					goto error;
 				}
 				free(orig);
 				if (features & BTRFS_FEATURE_LIST_ALL) {
 					btrfs_list_all_fs_features(0);
-					exit(0);
+					goto success;
 				}
 				break;
 				}
@@ -1511,8 +1514,7 @@ int main(int argc, char **argv)
 			case 'V':
 				printf("mkfs.btrfs, part of %s\n",
 						PACKAGE_STRING);
-				exit(0);
-				break;
+				goto success;
 			case 'r':
 				source_dir = optarg;
 				source_dir_set = 1;
@@ -1547,7 +1549,7 @@ int main(int argc, char **argv)
 
 	if (source_dir_set && dev_cnt > 1) {
 		error("the option -r is limited to a single device");
-		exit(1);
+		goto error;
 	}
 
 	if (*fs_uuid) {
@@ -1555,11 +1557,11 @@ int main(int argc, char **argv)
 
 		if (uuid_parse(fs_uuid, dummy_uuid) != 0) {
 			error("could not parse UUID: %s", fs_uuid);
-			exit(1);
+			goto error;
 		}
 		if (!test_uuid_unique(fs_uuid)) {
 			error("non-unique UUID: %s", fs_uuid);
-			exit(1);
+			goto error;
 		}
 	}
 
@@ -1567,7 +1569,7 @@ int main(int argc, char **argv)
 		file = argv[optind++];
 		if (is_block_device(file) == 1)
 			if (test_dev_for_mkfs(file, force_overwrite))
-				exit(1);
+				goto error;
 	}
 
 	optind = saved_optind;
@@ -1602,7 +1604,7 @@ int main(int argc, char **argv)
 			if (metadata_profile != data_profile) {
 				error(
 	"with mixed block groups data and metadata profiles must be the same");
-				exit(1);
+				goto error;
 			}
 		}
 
@@ -1624,12 +1626,12 @@ int main(int argc, char **argv)
 
 	if (btrfs_check_nodesize(nodesize, sectorsize,
 				 features))
-		exit(1);
+		goto error;
 
 	if (sectorsize < sizeof(struct btrfs_super_block)) {
 		error("sectorsize smaller than superblock: %u < %zu",
 				sectorsize, sizeof(struct btrfs_super_block));
-		exit(1);
+		goto error;
 	}
 
 	/* Check device/block_count after the nodesize is determined */
@@ -1638,7 +1640,7 @@ int main(int argc, char **argv)
 			block_count);
 		error("minimum size for btrfs filesystem is %llu",
 			btrfs_min_dev_size(nodesize));
-		exit(1);
+		goto error;
 	}
 	for (i = saved_optind; i < saved_optind + dev_cnt; i++) {
 		char *path;
@@ -1648,20 +1650,20 @@ int main(int argc, char **argv)
 		if (ret < 0) {
 			error("failed to check size for %s: %s",
 				path, strerror(-ret));
-			exit (1);
+			goto error;
 		}
 		if (ret > 0) {
 			error("'%s' is too small to make a usable filesystem",
 				path);
 			error("minimum size for each btrfs device is %llu",
 				btrfs_min_dev_size(nodesize));
-			exit(1);
+			goto error;
 		}
 	}
 	ret = test_num_disk_vs_raid(metadata_profile, data_profile,
 			dev_cnt, mixed, ssd);
 	if (ret)
-		exit(1);
+		goto error;
 
 	dev_cnt--;
 
@@ -1674,7 +1676,7 @@ int main(int argc, char **argv)
 		fd = open(file, O_RDWR);
 		if (fd < 0) {
 			error("unable to open %s: %s", file, strerror(errno));
-			exit(1);
+			goto error;
 		}
 		ret = btrfs_prepare_device(fd, file, &dev_block_count,
 				block_count,
@@ -1682,22 +1684,21 @@ int main(int argc, char **argv)
 				(discard ? PREP_DEVICE_DISCARD : 0) |
 				(verbose ? PREP_DEVICE_VERBOSE : 0));
 		if (ret) {
-			close(fd);
-			exit(1);
+			goto error;
 		}
 		if (block_count && block_count > dev_block_count) {
 			error("%s is smaller than requested size, expected %llu, found %llu",
 					file,
 					(unsigned long long)block_count,
 					(unsigned long long)dev_block_count);
-			exit(1);
+			goto error;
 		}
 	} else {
 		fd = open(file, O_CREAT | O_RDWR,
 				S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
 		if (fd < 0) {
 			error("unable to open %s: %s", file, strerror(errno));
-			exit(1);
+			goto error;
 		}
 
 		source_dir_size = size_sourcedir(source_dir, sectorsize,
@@ -1707,7 +1708,7 @@ int main(int argc, char **argv)
 		ret = zero_output_file(fd, block_count);
 		if (ret) {
 			error("unable to zero the output file");
-			exit(1);
+			goto error;
 		}
 		/* our "device" is the new image file */
 		dev_block_count = block_count;
@@ -1717,7 +1718,7 @@ int main(int argc, char **argv)
 	if (dev_block_count < BTRFS_MKFS_SYSTEM_GROUP_SIZE) {
 		error("device is too small to make filesystem, must be at least %llu",
 				(unsigned long long)BTRFS_MKFS_SYSTEM_GROUP_SIZE);
-		exit(1);
+		goto error;
 	}
 
 	if (group_profile_max_safe_loss(metadata_profile) <
@@ -1736,41 +1737,42 @@ int main(int argc, char **argv)
 	ret = make_btrfs(fd, &mkfs_cfg);
 	if (ret) {
 		error("error during mkfs: %s", strerror(-ret));
-		exit(1);
+		goto error;
 	}
 
 	fs_info = open_ctree_fs_info(file, 0, 0, 0,
 			OPEN_CTREE_WRITES | OPEN_CTREE_FS_PARTIAL);
 	if (!fs_info) {
 		error("open ctree failed");
-		close(fd);
-		exit(1);
+		goto error;
 	}
+	close(fd);
+	fd = -1;
 	root = fs_info->fs_root;
 	fs_info->alloc_start = alloc_start;
 
 	ret = create_metadata_block_groups(root, mixed, &allocation);
 	if (ret) {
 		error("failed to create default block groups: %d", ret);
-		exit(1);
+		goto error;
 	}
 
 	trans = btrfs_start_transaction(root, 1);
 	if (!trans) {
 		error("failed to start transaction");
-		exit(1);
+		goto error;
 	}
 
 	ret = create_data_block_groups(trans, root, mixed, &allocation);
 	if (ret) {
 		error("failed to create default data block groups: %d", ret);
-		exit(1);
+		goto error;
 	}
 
 	ret = make_root_dir(trans, root);
 	if (ret) {
 		error("failed to setup the root directory: %d", ret);
-		exit(1);
+		goto error;
 	}
 
 	ret = btrfs_commit_transaction(trans, root);
@@ -1782,7 +1784,7 @@ int main(int argc, char **argv)
 	trans = btrfs_start_transaction(root, 1);
 	if (!trans) {
 		error("failed to start transaction");
-		exit(1);
+		goto error;
 	}
 
 	if (dev_cnt == 0)
@@ -1799,7 +1801,7 @@ int main(int argc, char **argv)
 		fd = open(file, O_RDWR);
 		if (fd < 0) {
 			error("unable to open %s: %s", file, strerror(errno));
-			exit(1);
+			goto error;
 		}
 		ret = btrfs_device_already_in_root(root, fd,
 						   BTRFS_SUPER_INFO_OFFSET);
@@ -1815,8 +1817,7 @@ int main(int argc, char **argv)
 				(zero_end ? PREP_DEVICE_ZERO_END : 0) |
 				(discard ? PREP_DEVICE_DISCARD : 0));
 		if (ret) {
-			close(fd);
-			exit(1);
+			goto error;
 		}
 
 		ret = btrfs_add_to_fsid(trans, root, fd, file, dev_block_count,
@@ -1872,7 +1873,7 @@ raid_groups:
 			goto out;
 		}
 
-		ret = make_image(source_dir, root, fd);
+		ret = make_image(source_dir, root);
 		if (ret) {
 			error("error wihle filling filesystem: %d", ret);
 			goto out;
@@ -1940,4 +1941,12 @@ out:
 	free(label);
 
 	return !!ret;
+error:
+	if (fd > 0)
+		close(fd);
+
+	free(label);
+	exit(1);
+success:
+	exit(0);
 }
